from openpyxl import Workbook
from openpyxl.styles import PatternFill, Font, Alignment


def _BG(color: str):
    return PatternFill(fgColor=color, fill_type="solid")


class ExcelExporter:
    # Class attributes for styling
    sheet_name = "Exported Data"
    export_name = "export"

    row_height = 20
    column_width = 15

    horizontal_alignment = "center"
    vertical_alignment = "center"

    header_style = {
        "font": Font(bold=True, color="FFFFFF"),
        "fill": _BG("4F81BD"),
        "alignment": Alignment(
            horizontal=horizontal_alignment, vertical=vertical_alignment
        ),
    }

    row_styles = [
        {
            "fill": _BG("D9E1F2"),
            "alignment": Alignment(
                horizontal=horizontal_alignment, vertical=vertical_alignment
            ),
        },
        {
            "fill": _BG("FFFFFF"),
            "alignment": Alignment(
                horizontal=horizontal_alignment, vertical=vertical_alignment
            ),
        },
    ]

    def get_row_style(self, item: dict = None, index: int = None):
        assert (item is not None) or (
            index is not None
        ), "One of the arguments must be given"
        return self.row_styles[index % len(self.row_styles)]

    def get_row_height(self, item: dict = None, index: int = None):
        return self.row_height

    def get_column_width(self, item: dict = None, index: int = None):
        return self.column_width

    def get_cell_style(self, cell_column: str, cell_value: str, item: dict = None):
        return None

    def __init__(self, data, header_names, beautify_headers=False):
        self.data = data  # List of dictionaries representing rows
        self.headers = self._normalize_headers(header_names, beautify_headers)

    def _normalize_headers(self, header_names, beautify):
        if isinstance(header_names, list):
            return {
                name: self._beautify_header(name) if beautify else name
                for name in header_names
            }
        return header_names

    def _beautify_header(self, name):
        return " ".join(
            word.capitalize()
            for word in name.replace("_", " ").replace("-", " ").split()
        )

    @property
    def header_names(self):
        return list(self.headers.values())

    @property
    def header_fields(self):
        return list(self.headers.keys())

    def create_workbook(self):
        wb = Workbook()
        ws = wb.active
        ws.title = self.sheet_name

        # Set column widths
        for col_idx in range(1, len(self.header_names) + 1):
            ws.column_dimensions[chr(64 + col_idx)].width = self.get_column_width()

        # Write header row
        for col_idx, header in enumerate(self.header_names, start=1):
            cell = ws.cell(row=1, column=col_idx, value=header)
            cell.font = self.header_style["font"]
            cell.fill = self.header_style["fill"]
            cell.alignment = self.header_style["alignment"]

        # Write data rows
        for row_idx, row_data in enumerate(self.data, start=2):
            row_style = self.get_row_style(index=row_idx)
            for col_idx, field in enumerate(self.header_fields, start=1):
                # Set value of the cell
                value = row_data.get(field, "")
                cell = ws.cell(row=row_idx, column=col_idx, value=value)
                # Merge row style with cell style
                cell_style = self.get_cell_style(field, value, row_data) or {}
                final_style = {**row_style, **cell_style}
                # Apply cell styles
                cell.font = final_style.get("font")
                cell.fill = final_style.get("fill")
                cell.alignment = final_style.get("alignment")

            ws.row_dimensions[row_idx].height = self.get_row_height()

        return wb

    def save(self, filename=None):
        wb = self.create_workbook()
        name = filename or self.export_name
        if not name.endswith(".xlsx"):
            name += ".xlsx"
        wb.save(name)
